#!/bin/sh
# -*- tcl -*- \
SCRIPTNAME="$0"; export SCRIPTNAME; exec wish -f $SCRIPTNAME $*

#####################################################################
# This code is written and copyrighted by Clif Flynt, 1999

# It is licenced for public use with these terms:

#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.

#  You may examine and modify the code to your heart's content.

#  You may not claim to have authored the code (aside from modifications
#  you may have made.)

#  You may not distribute the code without this license agreement

#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Clif Flynt.

#  Maintenance, extension, modification, etc can be performed by:

#   Flynt Consulting Services,
#   9300 Fleming Rd.
#   Dexter, MI  48130

#   Contact: clif@cflynt.com
#####################################################################

################################################################
# proc notice {text}--
#    Put a short notice on the screen, just *there*, not placed anyplace
#    in particular.
# Arguments
#   The text for this notice.
# 
# Results
#   A new little window with a label and a cancel button.
# 
proc notice {text} {
    toplevel .t
    label .t.l1 -text $text
    button .t.b -text "CANCEL" -command "destroy .t"
    
    pack .t.l1 -side top
    pack .t.b -side top
}



################################################################
# proc canvasExtract {cvs}--
#    Extract the info about all the items in the canvas
# Arguments
#   cvs         The name of the canvas to extract items from

# Results
#   A list of lists
#   Each list element is a list of parameters for a cavas create command.

proc canvasExtract {cvs} {

  # Loop through all the items defined in this canvas by ID number

  foreach item [$cvs find all] {
    # Determine the coordinates and type, and use them to initialize the lst

    set lst "[$cvs type $item] [$cvs coords $item]"

    # Loop through the configuration options, and add the required text
    # to the command.

    foreach cfg [$cvs itemconfigure $item] {
       lappend lst [lindex $cfg 0] [lindex $cfg 4]
    }
  # Add this list to our list of commands
  lappend cmds $lst
  }
  return $cmds
}



################################################################
# proc configController {ct}--
#    Builds a configuration widget and displays it.
# Arguments
#   ct		The number of this controller in the canvas
# 
# Results
#   A new window appears, with bindings for the done and cancel keys
# 
proc configController {ct} {
   global Ctrl
   global Label ID Type
   
   toplevel .t
   set row 0;
   
   foreach l {Label ID Type} {
       label .t.l$l -text $l
       entry .t.e$l -textvar $l
       grid .t.l$l -row $row -column 0
       grid .t.e$l -row $row -column 1

       # Initialize the entry widget if this is previously defined.

       if {[info exists Ctrl($ct.$l)]} {
           set $l $Ctrl($ct.$l)
       } else {
           set $l ""
       }
       incr row
   }
   button .t.b1 -text "cancel" -command "destroy .t"
   button .t.b2 -text "OK" -command "setController $ct; destroy .t"
   incr row
   grid .t.b1 -row $row -column 0
   grid .t.b2 -row $row -column 1
}


################################################################
# proc setController {ct}--
#   Sets values for a controller, and bindings to turn on/off
# Arguments
#   ct		The identifier for this controller (canvas ID)
# 
# Results
#   Sets bindings, and sets values in Ctrl array to describe this controller
# 
proc setController {ct} {
    global Label ID Type Ctrl;
    $Ctrl(canvas) bind $ct <ButtonPress-1> "exec /usr/local/bin/br $ID on"
    $Ctrl(canvas) bind $ct <ButtonPress-3> "exec /usr/local/bin/br $ID off"

    if {$Ctrl($ct.X) < [expr [.c cget -width] / 2]} {
        set anchor w
	set xpos [expr 22 + $Ctrl($ct.X)]
    } else {
        set anchor e
	set xpos [expr $Ctrl($ct.X) - 5]
    }

    $Ctrl(canvas) bind $ct <Enter> "\
        after 2 {$Ctrl(canvas) create text $xpos $Ctrl($ct.Y) \
          -tag tmpLabel -text {$Label} -anchor $anchor}"

    $Ctrl(canvas) bind $ct <Leave> {after 2 {$Ctrl(canvas) delete tmpLabel}}
    
    array set Ctrl [list $ct.ID $ID $ct.Label $Label $ct.Type $Type]
}

################################################################
# proc placeController {x y}--
#    Places a controller icon on the map, and sets the initial binding
# Arguments
#   X 	X location for upper left corner of icon
#   Y 	Y location for upper left corner of icon
# 
# Results
#   A rectangle is created and tagged with its canvas item number
#   A binding is added to this canvas item
#   The Ctrl array is updated with the X, Y location for this item.
# 
proc placeController {x y} {
   global Ctrl

    set ct [$Ctrl(canvas) create rectangle  $x $y \
        [expr $x+15] [expr $y+15] -fill red]
	
    $Ctrl(canvas) itemconfigure $ct -tag "it,$ct"

    lappend Ctrl(devices) it,$ct

    $Ctrl(canvas) raise it,$ct
    $Ctrl(canvas) bind it,$ct <Control-ButtonPress-1> "configController it,$ct"
    set Ctrl(it,$ct.X) $x
    set Ctrl(it,$ct.Y) $y
}

################################################################
# proc addController {}--
#    Sets up binding to add a controller icon to the screen.
# Arguments
#   None
# 
# Results
#   sets a notice to show it is active, and sets a binding to place a 
#   controller icon on the map
# 
proc addController {} {
   global Ctrl

   notice "Click the location for the new controller"
   
   bind $Ctrl(canvas) <ButtonPress-1> {
         placeController %x %y; 
	 destroy .t
	 bind $Ctrl(canvas) <ButtonPress-1> ""
	}
}


################################################################
# proc saveState {fileName}--
#    Saves the state of the canvas - all icons, maps and bindings.
# Arguments
#   fileName	The name of the file to save the data to.
# 
# Results
#   Replaces any existing fileName with the current values.
# 
proc saveState {fileName} {
    global Ctrl
    
    set outfl [open $fileName "w"]
    puts $outfl "array set Ctrl [list [array get Ctrl]]"


   foreach l [canvasExtract $Ctrl(canvas)] {
      puts $outfl "$Ctrl(canvas) create $l"
   }
   
  foreach item [$Ctrl(canvas) find all] {
      foreach binding [$Ctrl(canvas) bind it,$item] {
          puts $outfl "$Ctrl(canvas) bind it,$item $binding \
	  	[list [$Ctrl(canvas) bind it,$item $binding]]"
      }
  }

    close $outfl
}

#..................................................................

set Ctrl(Rev) {$RCSfile: control.tcl,v $ $Revision: 1.5 $}

if {$argc != 1} {
    puts "?wish? control.tcl image.gif"
    exit
}

set name [file root [lindex $argv 0]]
set path [file dirname [lindex $argv 0]]

set blueprint [image create photo blueprint -file [lindex $argv 0]]

set Ctrl(canvas) [canvas .c -height [image height blueprint] \
			     -width [image width blueprint] ]

grid $Ctrl(canvas) -row 0 -column 0 -columnspan 6

button .b1 -text "Add Controller" -command addController
grid .b1 -row 1 -column 0

button .b2 -text "Save" -command "saveState [file join $path $name.dat]"
grid .b2 -row 1 -column 1

button .b3 -text "Exit" -command exit
grid .b3 -row 1 -column 2

# if a save file exists, source it, else just display the blueprint


if {[file exists [file join $path $name.dat]]} {
    source [file join $path $name.dat]
} else {
    $Ctrl(canvas) create image 1 1 -image $blueprint -anchor nw
}

set rev [string range $Ctrl(Rev) [expr [string last ":" $Ctrl(Rev) ] +1] \
	[string last " " $Ctrl(Rev) ]]

wm title . "Household X-10 control                                Revision: $rev"

